@@ -4,10 +4,11 @@ from importlib_metadata import version
 SDK_NAME = "unleash-client-python"
 SDK_VERSION = version("UnleashClient")
 REQUEST_TIMEOUT = 30
+REQUEST_RETRIES = 3
 METRIC_LAST_SENT_TIME = "mlst"
 
 # =Unleash=
-APPLICATION_HEADERS = {"Content-Type": "application/json"}
+APPLICATION_HEADERS = {"Content-Type": "application/json", "Unleash-Client-Spec": "4.2.0"}
 DISABLED_VARIATION = {
     'name': 'disabled',
     'enabled': False
